using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.IO;

//****************************************
//Author: Alex Schafer
//Copyright (c) 2009
//See end of file for terms of use.  
//***************************************
//Version 1.0 - Initial Release

//************** N A M E S P A C E ****************************************
namespace PropellerUtilities.Font {
	//*********************************************************************
	// AIGenericWriter Class
	//*********************************************************************	
	public class AIGenericWriter {					
		/// <sumary>
		/// Build the static header for the output file
		/// </sumary>		
		private static string BuildHeaderText(int fontChars) {
			string headerText = "'' Font generated by FontGenerator\n";
			headerText += "'' compatable with AIGeneric 1.2\n\n";
			headerText += "CON\n\n";
			headerText += "        AS_VIEWED               = true\n";
			headerText += "        FONT_CHARS              = "+fontChars+"\n\n";
			
			headerText += "PUB GetPtrToFontTable\n\n";

			headerText += "        result                  := @fonttab\n\n";

			headerText += "pub define(c,c0,c1,c2,c3,c4,c5,c6,c7) | p \n";
			headerText += " p:=@fonttab+(c<<3)\n";
			headerText += " byte[p][0]:=c0\n";
			headerText += " byte[p][1]:=c1\n";
			headerText += " byte[p][2]:=c2\n";
			headerText += " byte[p][3]:=c3\n";
			headerText += " byte[p][4]:=c4\n";
			headerText += " byte[p][5]:=c5\n";
			headerText += " byte[p][6]:=c6\n";
			headerText += " byte[p][7]:=c7\n\n";
			headerText += "DAT\n\n";
			
			return headerText;
		}
		
		//*********** P U B L I C   F U N C T I O N S  ( M E T H O D S ) ******

		public static void Write (Bitmap bitmap, string outputFileName) {						
			try {
				StreamWriter streamWriter = new StreamWriter(outputFileName);
						
				//the assumption is that each are 8x8 character blocks
				int totalCharacterRows = bitmap.Height / 8;
				int totalFontChars = totalCharacterRows * 32;
								
				streamWriter.Write(BuildHeaderText(totalFontChars));
				int currentCharacter = 0;
				for (int characterRow=0; characterRow < totalCharacterRows; characterRow++) {
					for (int characterLineWidth=0; characterLineWidth <32; characterLineWidth++) {

						for (int characterHeight = (characterRow * (int)8); characterHeight < (characterRow * (int)8) + (int)8; characterHeight++) {
							if ((currentCharacter == 0) && (characterHeight == (characterRow * (int)8))) {
								streamWriter.Write ("fonttab "); 
							} else {
								streamWriter.Write ("        "); 
							}

							streamWriter.Write ("byte byte ");
							string byteString = "";
							string picString = "";
							for (int characterWidth = (characterLineWidth * (int)8); characterWidth < (characterLineWidth * (int)8)+(int)8; characterWidth++) {
												
								Color pixelColor = bitmap.GetPixel(characterWidth, characterHeight);
											
								if ((pixelColor.R == 255) && (pixelColor.G == 255) && (pixelColor.B == 255)) {
									picString += "#";
									byteString += "1";													
								} else {
									picString +=  ".";
									byteString += "0";
								}
												
							}
							//special case: put in AS_VIEWED if we are on 'space' (32)
							if ((currentCharacter == 32) && (characterHeight == (characterRow * (int)8))) {
								byteString = "AS_VIEWED";
							} else {
								byteString = "%" + byteString;
							}
											
							streamWriter.Write(byteString + "     ' "+ picString);
							if (characterHeight == (characterRow * (int)8)) streamWriter.Write (" $" + FontUtilities.ToHexString(currentCharacter)); 
								streamWriter.WriteLine("");
							}
							streamWriter.WriteLine("");
							currentCharacter++;
						}
					}
					streamWriter.WriteLine("' End of data");
					streamWriter.Close();
					streamWriter = null;
						
			} catch (Exception e){
				throw new FontException("Error in creating output file " + outputFileName + " - " + e.ToString());
			}								
		}
	}
}
/*
+------------------------------------------------------------------------------------------------------------------------------+
                                                   TERMS OF USE: MIT License                                                                                                              
+------------------------------------------------------------------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                              
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                              
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
+------------------------------------------------------------------------------------------------------------------------------+
*/
